#include "scr.h"
#include "res.h"
#include "libmy/lex.h"
#include "libmy/mem.h"
#include "libmy/bg.h"
#include "libmy/spr.h"
#include "libmy/key.h"


//---------------------------------------------------------------------------
ST_STAGE  Stage;
ST_PLAYER Player;


const ST_STAGE_DB StageDb[STAGE_MAX_CNT] = {
	{ &dat_stage1_txt },
	{ &dat_stage2_txt },
	{ &dat_stage3_txt },
};

//---------------------------------------------------------------------------
EWRAM_CODE void ScrInit()
{
	_Memset(&Stage, 0x00, sizeof(ST_STAGE));
	_Memset(&Player, 0x00, sizeof(ST_PLAYER));

	BgSetDat(BG_SYS, (u16*)&bg_asciiTiles, bg_asciiTilesLen, (u16*)&bg_asciiPal, bg_asciiPalLen);
	SprSetDat((u16*)&spr_chrTiles, spr_chrTilesLen, (u16*)&spr_chrPal, spr_chrPalLen);

	Player.sprX = 240 / 2 - 32 / 2;
	Player.sprY = 160 / 2 - 32 / 2;
}
//---------------------------------------------------------------------------
IWRAM_CODE void ScrExecMove()
{
	Stage.hofs += Stage.dirX * 2;
	Stage.vofs += Stage.dirY * 2;

	BgSetHofs2(Stage.hofs);
	BgSetVofs2(Stage.vofs);

	if(++Stage.moveCnt >= 16)
	{
		Player.x += Stage.dirX;
		Player.y += Stage.dirY;

		if(Stage.moveChr == STAGE_CHR_BLOCK || Stage.moveChr == STAGE_CHR_BLOCK_ON)
		{
			ScrCalcChrReverse(Player.x + Stage.dirX, Player.y + Stage.dirY);
		}

		Stage.isMove = FALSE;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void ScrLoadMap(u16 mapNo)
{
	if(Stage.pMap != NULL)
	{
		MemFree(Stage.pMap);
	}


	s16 fd = LexOpen(StageDb[mapNo].pTxt);
	u16 idx;

	Stage.cx = LexGetSafeNum(fd);
	Stage.cy = LexGetSafeNum(fd);
	Stage.pMap = MemCalloc(Stage.cx * Stage.cy * 2);

	for(idx=0; idx<Stage.cx * Stage.cy; idx++)
	{
		u16 chr = LexGetSafeChr(fd);

		if(chr == STAGE_CHR_PLAYER)
		{
			chr = STAGE_CHR_SPACE;
			ScrSetPlayer(idx);
		}
		else if(chr == STAGE_CHR_PLAYER_ON)
		{
			chr = STAGE_CHR_SPACE_ON;
			ScrSetPlayer(idx);
		}

		Stage.pMap[idx] = chr;
	}

	LexClose(fd);
//	ScrDebugMap();
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScrLoadTile()
{
	BgSetDat(BG_STAGE, (u16*)&bg_stageTiles, bg_stageTilesLen, (u16*)&bg_stagePal, bg_stagePalLen);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScrDrawTitle()
{
	BgSetDat(BG_STAGE, (u16*)&bg_titleTiles, bg_titleTilesLen, (u16*)&bg_titlePal, bg_titlePalLen);
	BgDrawCap(BG_STAGE);

	BgDrawStr(BG_ASCII, 9, 13, "PUSH A BUTTON");
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScrDrawSelect()
{
	BgSetDat(BG_STAGE, (u16*)&bg_selectTiles, bg_selectTilesLen, (u16*)&bg_selectPal, bg_selectPalLen);
	BgDrawCap(BG_STAGE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScrDrawSelectNum(u16 num)
{
	char str[32];

	_Sprintf(str, "<- %d ->", num);
	BgDrawStr(BG_ASCII, 11, 11, str);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScrDrawGameClear()
{
	BgSetDat(BG_STAGE, (u16*)&bg_clearTiles, bg_clearTilesLen, (u16*)&bg_clearPal, bg_clearPalLen);
	BgDrawCap(BG_STAGE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScrDrawCls()
{
	BgSetHofs2(0);
	BgSetVofs2(0);
	BgDrawCls(BG_ASCII);
	BgDrawCls(BG_STAGE);

	SprSetCls();
}
//---------------------------------------------------------------------------
IWRAM_CODE void ScrDrawStage()
{
	u16 x, y;

	for(y=0; y<Stage.cy; y++)
	{
		for(x=0; x<Stage.cx; x++)
		{
			u16 chr = ScrGetChr(x, y);
			u16 tileNo;

			switch(chr)
			{
			default:
			case STAGE_CHR_NONE:
				tileNo = BG_TILE_BLACK;
				break;

			case STAGE_CHR_WALL:
				tileNo = BG_TILE_WALL;
				break;

			// 
			case STAGE_CHR_SPACE:
				tileNo = BG_TILE_SPACE;
				break;

			// uꏊ
			case STAGE_CHR_SPACE_ON:
				tileNo = BG_TILE_SPACE_ON;
				break;

			// ו
			case STAGE_CHR_BLOCK:
				tileNo = BG_TILE_BLOCK;
				break;

			// uꏊ{ו
			case STAGE_CHR_BLOCK_ON:
				tileNo = BG_TILE_BLOCK_ON;
				break;
			}

			BgDrawChr4(BG_STAGE, x, y, tileNo);
		}
	}

	BgSetHofs2(Stage.hofs);
	BgSetVofs2(Stage.vofs);
}
//---------------------------------------------------------------------------
IWRAM_CODE void ScrDrawChr()
{
	u16 tileNo = SPR_TILE_NONE;
	s16 x = 0;
	s16 y = 0;

	// ړ̂݁AXvCg̉ו\܂
	if(Stage.isMove == TRUE)
	{
		if(Stage.moveChr == STAGE_CHR_BLOCK)
		{
			tileNo = SPR_TILE_BLOCK;
		}
		else if(Stage.moveChr == STAGE_CHR_BLOCK_ON)
		{
			tileNo = SPR_TILE_BLOCK_ON;
		}

		x = Stage.dirX * 32;
		y = Stage.dirY * 32;
	}

	SprSetChr(1, Player.sprX + x, Player.sprY + y, tileNo);
	SprSetChr(0, Player.sprX, Player.sprY, SPR_TILE_PLAYER);
}
//---------------------------------------------------------------------------
IWRAM_CODE void ScrCalcMove(u16 key)
{
	s16 dirX, dirY;

	if(key & KEY_UP)
	{
		dirX = 0;
		dirY = -1;
	}
	else if(key & KEY_DOWN)
	{
		dirX = 0;
		dirY = 1;
	}
	else if(key & KEY_LEFT)
	{
		dirX = -1;
		dirY = 0;
	}
	else if(key & KEY_RIGHT)
	{
		dirX = 1;
		dirY = 0;
	}

	Stage.isMove  = TRUE;
	Stage.moveCnt = 0;
	Stage.dirX    = dirX;
	Stage.dirY    = dirY;
	Stage.moveChr = ScrGetChr(Player.x + dirX, Player.y + dirY);

	if(Stage.moveChr == STAGE_CHR_BLOCK || Stage.moveChr == STAGE_CHR_BLOCK_ON)
	{
		ScrCalcChrReverse(Player.x + dirX, Player.y + dirY);
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void ScrCalcChrReverse(s16 x, s16 y)
{
	// LN^f[^𔽓]܂iړpj

	u16 chr = ScrGetChr(x, y);

	if(chr == STAGE_CHR_NONE)
	{
		return;
	}

	switch(chr)
	{
	//   ו
	case STAGE_CHR_SPACE:
		chr = STAGE_CHR_BLOCK;
		break;

	// ו  
	case STAGE_CHR_BLOCK:
		chr = STAGE_CHR_SPACE;
		break;

	// uꏊ  uꏊ{ו
	case STAGE_CHR_SPACE_ON:
		chr = STAGE_CHR_BLOCK_ON;
		break;

	// uꏊ{ו  uꏊ
	case STAGE_CHR_BLOCK_ON:
		chr = STAGE_CHR_SPACE_ON;
		break;
	}

	ScrSetChr(x, y, chr);
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 ScrGetMaxStageCnt()
{
	return STAGE_MAX_CNT;
}
//---------------------------------------------------------------------------
IWRAM_CODE u16 ScrGetChr(s16 x, s16 y)
{
	if(x < 0) return STAGE_CHR_NONE;
	if(y < 0) return STAGE_CHR_NONE;
	if(x > Stage.cx) return STAGE_CHR_NONE;
	if(y > Stage.cy) return STAGE_CHR_NONE;

	return Stage.pMap[Stage.cx * y + x];
}
//---------------------------------------------------------------------------
IWRAM_CODE void ScrSetChr(s16 x, s16 y, u16 chr)
{
	if(x < 0) return;
	if(y < 0) return;
	if(x > Stage.cx) return;
	if(y > Stage.cy) return;

	Stage.pMap[Stage.cx * y + x] = chr;
}
//---------------------------------------------------------------------------
IWRAM_CODE void ScrSetPlayer(u16 idx)
{
	Player.x   = Mod(idx, Stage.cx);
	Player.y   = Div(idx, Stage.cx);
	Stage.hofs = ((Player.x - 3) * 32) - 8;
	Stage.vofs = ((Player.y - 2) * 32) - 0;
}
//---------------------------------------------------------------------------
IWRAM_CODE bool ScrIsPlayerMove(u16 key)
{
	if(key & KEY_UP)
	{
		return ScrIsPlayerMove2(0, -1);
	}
	else if(key & KEY_DOWN)
	{
		return ScrIsPlayerMove2(0, 1);
	}
	else if(key & KEY_LEFT)
	{
		return ScrIsPlayerMove2(-1, 0);
	}
	else if(key & KEY_RIGHT)
	{
		return ScrIsPlayerMove2(1, 0);
	}

	return FALSE;
}
//---------------------------------------------------------------------------
IWRAM_CODE bool ScrIsPlayerMove2(s16 x, s16 y)
{
	u16 chr  = ScrGetChr(Player.x + x    , Player.y + y);
	u16 chr2 = ScrGetChr(Player.x + x + x, Player.y + y + y);

	if(chr == STAGE_CHR_SPACE || chr == STAGE_CHR_SPACE_ON)
	{
		return TRUE;
	}

	if(chr == STAGE_CHR_BLOCK || chr == STAGE_CHR_BLOCK_ON)
	{
		if(chr2 == STAGE_CHR_SPACE || chr2 == STAGE_CHR_SPACE_ON)
		{
			return TRUE;
		}
	}

	return FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool ScrIsMoveEnd()
{
	return Stage.isMove;
}
//---------------------------------------------------------------------------
IWRAM_CODE bool ScrIsGameEnd()
{
	u16 i;

	for(i=0; i<Stage.cx * Stage.cy; i++)
	{
		if(Stage.pMap[i] == STAGE_CHR_SPACE_ON)
		{
			return FALSE;
		}
	}

	return TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScrDebugMap()
{
	u16 x = 0;
	u16 i;

	TRACEOUT("------------------------------\n"); 
	TRACEOUT("ScrDebugMap\n");

	for(i=0; i<Stage.cx * Stage.cy; i++)
	{
		TRACEOUT("%c", Stage.pMap[i]);

		if(x++ >= Stage.cx-1)
		{
			TRACEOUT("\n");
			x = 0;
		}
	}
}
