#include "sys.h"
#include "libmy/libmy.h"
#include "irq_arm.h"
#include "game.h"


//---------------------------------------------------------------------------
ST_SYS Sys;


//---------------------------------------------------------------------------
EWRAM_CODE void SysInit()
{
	REG_WSCNT = 0x4317;
	Sys.act = SYS_INIT;
}
//---------------------------------------------------------------------------
IWRAM_CODE void SysExec()
{
	for(;;)
	{
		switch(Sys.act)
		{
		case SYS_INIT:
			SysExecInit();
			break;

		case SYS_RESET:
			SysExecReset();
			break;

		case SYS_RUN:
			SysExecRun();
			break;

		default:
			SystemError("[Err] SysExec Sys.act=%d\n", Sys.act);
			break;
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void SysExecInit()
{
	MtInit();
	MemInit();
//	GbfsInit();

	BgInit();
//	Mode3Init();

	SprInit();
	KeyInit();
	DmaInit();
	LexInit();
	SndInit();
	FadeInit();
	DebugInit();

	IntrInit();
	IrqInit();
	IntrStart();

	Sys.act = SYS_RESET;
}
//---------------------------------------------------------------------------
IWRAM_CODE void SysExecReset()
{
	GameInit();

	Sys.act = SYS_RUN;
}
//---------------------------------------------------------------------------
IWRAM_CODE void SysExecRun()
{
	for(;;)
	{
		SystemCall(5);

		// Dma
		DmaTransfer();

		// Draw
		GameDraw();
		FadeDraw();
		DebugDraw();

		// Exec
		BgExec();
		SprExec();
		KeyExec();
		GameExec();
		DebugExec();
	}
}
